package djh.openEJBExample.ejb;

import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import djh.openEJBExample.entities.Employee;
import djh.openEJBExample.service.EmployeeDAO;

@Stateless
public class EmployeeDAOEJB implements EmployeeDAO {

	@PersistenceContext(unitName = "employees")
	private EntityManager em;

	public Employee create(Employee employee) {
		em.persist(employee);
		em.flush();
		return employee;
	}

	public Employee getById(int id) {
		Employee employee = em.find(Employee.class, id);
		return employee;
	}

	public List<Employee> getAll() {
		TypedQuery<Employee> query = em.createQuery(
				"select emp from Employee emp", Employee.class);
		List<Employee> employees = query.getResultList();
		return employees;
	}

	public List<Employee> getByLastName(String lastName) {
		TypedQuery<Employee> query = em.createQuery(
				"select emp from Employee emp where emp.lastName=?1",
				Employee.class);
		query.setParameter(1, lastName);
		List<Employee> employees = query.getResultList();
		return employees;
	}

	public Employee update(Employee employee) {
		employee = em.merge(employee);
		em.flush();
		return employee;
	}

	public void remove(Employee employee) {
		employee = em.merge(employee);
		em.remove(employee);
		em.flush();
	}

}
