package djh.openEJBExample.service;

import javax.ejb.EJB;
import javax.transaction.UserTransaction;
import javax.annotation.Resource;
import java.util.List;

import org.junit.Test;
import org.junit.Assert;

import org.apache.openejb.api.LocalClient;

import djh.openEJBExample.openEJB.OpenEJBTestBase;
import djh.openEJBExample.entities.Employee;

@LocalClient
public class EmployeeDAOTest extends OpenEJBTestBase {

	@Resource
	private UserTransaction userTX;

	@EJB
	private EmployeeDAO dao;

	@Test
	public void testResourceInjection() {
		Assert.assertNotNull(userTX);
		Assert.assertNotNull(dao);
	}

	@Test
	public void testCRUD() throws Exception {
		Employee employee = new Employee();
		employee.setLastName("Doe");
		employee.setFirstName("John");

		employee = dao.create(employee);

		int id = employee.getId();

		Employee employee2 = dao.getById(id);
		Assert.assertNotNull(employee2);

		employee2.setLastName("Roe");
		employee2.setFirstName("Richard");
		employee2 = dao.update(employee2);

		List<Employee> roes = dao.getByLastName("Roe");
		Assert.assertNotNull(roes);
		Assert.assertTrue(roes.size() > 0);

		dao.remove(employee2);

		Employee employee3 = dao.getById(id);
		Assert.assertNull(employee3);
	}

	@Test
	public void testStaleUpdate() {
		Employee employee = new Employee();
		employee.setLastName("Doe");
		employee.setFirstName("Jane");
		employee = dao.create(employee);
		int id = employee.getId();

		Employee employee2 = dao.getById(id);
		employee2.setFirstName("Jean");
		employee2 = dao.update(employee2);
		Assert.assertTrue(employee2.getVersion() > employee.getVersion());

		employee.setFirstName("Janet");
		try {
			employee = dao.update(employee);
			Assert.fail("Expected Exception");
		} catch (Exception e) {
			Assert.assertTrue(true);
		}
	}

	@Test
	public void testRollback() throws Exception {
		Employee employee = new Employee();
		employee.setLastName("Bert");
		employee.setFirstName("Dog");

		employee = dao.create(employee);
		int id = employee.getId();

		userTX.begin();

		employee.setFirstName("Rat");
		employee = dao.update(employee);

		Employee employee2 = dao.getById(id);
		Assert.assertEquals("Rat", employee2.getFirstName());
		// same context, same object
		Assert.assertTrue(employee2 == employee);

		userTX.rollback();

		Employee employee3 = dao.getById(id);
		Assert.assertEquals("Dog", employee3.getFirstName());
		// different context, different object
		Assert.assertTrue(employee3 != employee);
	}
}
